@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1998-2015, University of Amsterdam
+    Copyright (c)  1998-2025, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -37,9 +38,15 @@
           [ edit/1,                     % +Spec
             edit/0
           ]).
-:- use_module(library(lists), [append/3, member/2, nth1/3]).
-:- use_module(library(make), [make/0]).
-:- set_prolog_flag(generate_debug_info, false).
+:- autoload(library(lists), [member/2, append/3, select/3]).
+:- autoload(library(make), [make/0]).
+:- autoload(library(prolog_breakpoints), [breakpoint_property/2]).
+:- autoload(library(apply), [foldl/5, maplist/3, maplist/2]).
+:- use_module(library(dcg/high_order), [sequence/5]).
+:- autoload(library(readutil), [read_line_to_string/2]).
+
+
+% :- set_prolog_flag(generate_debug_info, false).
 
 /** <module> Editor interface
 
@@ -53,6 +60,8 @@ an editor.
     locate/3,                       % +Partial, -FullSpec, -Location
     locate/2,                       % +FullSpec, -Location
     select_location/3,              % +Pairs, +Spec, -Location
+    exists_location/1,              % +Location
+    user_select/2,                  % +Max, -I
     edit_source/1,                  % +Location
     edit_command/2,                 % +Editor, -Command
     load/0.                         % provides load-hooks
@@ -73,7 +82,8 @@ edit_no_trace(Spec) :-
     findall(Location-FullSpec,
             locate(Spec, FullSpec, Location),
             Pairs0),
-    merge_locations(Pairs0, Pairs),
+    sort(Pairs0, Pairs1),
+    merge_locations(Pairs1, Pairs),
     do_select_location(Pairs, Spec, Location),
     do_edit_source(Location).
 
@@ -104,49 +114,36 @@ edit :-
                  *            LOCATE            *
                  *******************************/
 
-%!  locate(+Spec, -FullSpec, -Location)
+%!  locate(+Spec, -FullSpec, -Location:dict)
 
-locate(FileSpec:Line, file(Path, line(Line)), [file(Path), line(Line)]) :-
+locate(FileSpec:Line, file(Path, line(Line)), #{file:Path, line:Line}) :-
     integer(Line), Line >= 1,
     ground(FileSpec),                      % so specific; do not try alts
     !,
-    locate(FileSpec, _, [file(Path)]).
+    locate(FileSpec, _, #{file:Path}).
 locate(FileSpec:Line:LinePos,
        file(Path, line(Line), linepos(LinePos)),
-       [file(Path), line(Line), linepos(LinePos)]) :-
+       #{file:Path, line:Line, linepos:LinePos}) :-
     integer(Line), Line >= 1,
     integer(LinePos), LinePos >= 1,
     ground(FileSpec),                      % so specific; do not try alts
     !,
-    locate(FileSpec, _, [file(Path)]).
-locate(Path, file(Path), [file(Path)]) :-
+    locate(FileSpec, _, #{file:Path}).
+locate(Path, file(Path), #{file:Path}) :-
     atom(Path),
-    exists_file(Path),
-    \+ exists_directory(Path).
-locate(Pattern, file(Path), [file(Path)]) :-
+    exists_file(Path).
+locate(Pattern, file(Path), #{file:Path}) :-
     atom(Pattern),
-    catch(expand_file_name(Pattern, Files), _, fail),
+    catch(expand_file_name(Pattern, Files), error(_,_), fail),
     member(Path, Files),
-    exists_file(Path),
-    \+ exists_directory(Path).
-locate(FileBase, file(File), [file(File)]) :-
+    exists_file(Path).
+locate(FileBase, file(File), #{file:File}) :-
     atom(FileBase),
-    absolute_file_name(FileBase,
-                       [ file_type(prolog),
-                         access(read),
-                         file_errors(fail)
-                       ],
-                       File),
-    \+ exists_directory(File).
-locate(FileSpec, file(File), [file(File)]) :-
-    catch(absolute_file_name(FileSpec,
-                             [ file_type(prolog),
-                               access(read),
-                               file_errors(fail)
-                             ],
-                             File),
-          _, fail).
-locate(FileBase, source_file(Path), [file(Path)]) :-
+    find_source(FileBase, File).
+locate(FileSpec, file(File), #{file:File}) :-
+    is_file_search_spec(FileSpec),
+    find_source(FileSpec, File).
+locate(FileBase, source_file(Path),  #{file:Path}) :-
     atom(FileBase),
     source_file(Path),
     file_base_name(Path, File),
@@ -154,7 +151,7 @@ locate(FileBase, source_file(Path), [file(Path)]) :-
     ->  true
     ;   file_name_extension(FileBase, _, File)
     ).
-locate(FileBase, include_file(Path), [file(Path)]) :-
+locate(FileBase, include_file(Path),  #{file:Path}) :-
     atom(FileBase),
     setof(Path, include_file(Path), Paths),
     member(Path, Paths),
@@ -174,22 +171,16 @@ locate(Name//DCGArity, FullSpec, Location) :-
         locate(Name/Arity, FullSpec, Location)
     ;   locate(Name/_, FullSpec, Location) % demand arity >= 2
     ).
-locate(Name/Arity, library(File), [file(PlPath)]) :-
+locate(Name/Arity, library(File),  #{file:PlPath}) :-
     atom(Name),
     '$in_library'(Name, Arity, Path),
-    (   absolute_file_name(library(.),
+    (   absolute_file_name(library(.), Dir,
                            [ file_type(directory),
                              solutions(all)
-                           ],
-                           Dir),
+                           ]),
         atom_concat(Dir, File0, Path),
         atom_concat(/, File, File0)
-    ->  absolute_file_name(Path,
-                           [ file_type(prolog),
-                             access(read),
-                             file_errors(fail)
-                           ],
-                           PlPath)
+    ->  find_source(Path, PlPath)
     ;   fail
     ).
 locate(Module:Name, Module:Name/Arity, Location) :-
@@ -209,14 +200,53 @@ locate(Spec, Spec, Location) :-
 include_file(Path) :-
     source_file_property(Path, included_in(_,_)).
 
+%!  is_file_search_spec(@Spec) is semidet.
+%
+%   True if Spec is valid pattern for absolute_file_name/3.
+
+is_file_search_spec(Spec) :-
+    compound(Spec),
+    compound_name_arguments(Spec, Alias, [Arg]),
+    is_file_spec(Arg),
+    user:file_search_path(Alias, _),
+    !.
+
+is_file_spec(Name), atom(Name) => true.
+is_file_spec(Name), string(Name) => true.
+is_file_spec(Term), cyclic_term(Term) => fail.
+is_file_spec(A/B) => is_file_spec(A), is_file_spec(B).
+is_file_spec(_) => fail.
+
+%!  find_source(++FileSpec, =File) is semidet.
+%
+%   Find a source file from FileSpec.  If FileSpec resolves to a .qlf
+%   file, File is the embedded `.pl` file (which may not exist).
+
+find_source(FileSpec, File) :-
+    catch(absolute_file_name(FileSpec, File0,
+                             [ file_type(prolog),
+                               access(read),
+                               file_errors(fail)
+                             ]),
+          error(_,_), fail),
+    prolog_source(File0, File).
+
+prolog_source(File0, File) :-
+    file_name_extension(_, Ext, File0),
+    user:prolog_file_type(Ext, qlf),
+    !,
+    '$qlf_module'(File0, Info),
+    File = Info.get(file).
+prolog_source(File, File).
+
 
 %!  locate(+Spec, -Location)
 %
 %   Locate object from the specified location.
 
-locate(file(File, line(Line)), [file(File), line(Line)]).
-locate(file(File), [file(File)]).
-locate(Module:Name/Arity, [file(File), line(Line)]) :-
+locate(file(File, line(Line)), #{file:File, line:Line}).
+locate(file(File), #{file:File}).
+locate(Module:Name/Arity, #{file:File, line:Line}) :-
     (   atom(Name), integer(Arity)
     ->  functor(Head, Name, Arity)
     ;   Head = _                    % leave unbound
@@ -235,25 +265,25 @@ locate(Module:Name/Arity, [file(File), line(Line)]) :-
     functor(Head, Name, Arity),     % bind arity
     predicate_property(Module:Head, file(File)),
     predicate_property(Module:Head, line_count(Line)).
-locate(module(Module), [file(Path)|Rest]) :-
+locate(module(Module), Location) :-
     atom(Module),
     module_property(Module, file(Path)),
     (   module_property(Module, line_count(Line))
-    ->  Rest = [line(Line)]
-    ;   Rest = []
+    ->  Location = #{file:Path, line:Line}
+    ;   Location = #{file:Path}
     ).
 locate(breakpoint(Id), Location) :-
     integer(Id),
     breakpoint_property(Id, clause(Ref)),
     (   breakpoint_property(Id, file(File)),
         breakpoint_property(Id, line_count(Line))
-    ->  Location = [file(File),line(Line)]
+    ->  Location =  #{file:File, line:Line}
     ;   locate(clause(Ref), Location)
     ).
-locate(clause(Ref), [file(File), line(Line)]) :-
+locate(clause(Ref), #{file:File, line:Line}) :-
     clause_property(Ref, file(File)),
     clause_property(Ref, line_count(Line)).
-locate(clause(Ref, _PC), [file(File), line(Line)]) :- % TBD: use clause
+locate(clause(Ref, _PC), #{file:File, line:Line}) :- % TBD: use clause
     clause_property(Ref, file(File)),
     clause_property(Ref, line_count(Line)).
 
@@ -279,18 +309,11 @@ do_edit_source(Location) :-             % hook
     !.
 do_edit_source(Location) :-             % PceEmacs
     current_prolog_flag(editor, Editor),
-    pceemacs(Editor),
+    is_pceemacs(Editor),
     current_prolog_flag(gui, true),
     !,
-    memberchk(file(File), Location),
-    (   memberchk(line(Line), Location)
-    ->  (   memberchk(linepos(LinePos), Location)
-        ->  Pos = (File:Line:LinePos)
-        ;   Pos = (File:Line)
-        )
-    ;   Pos = File
-    ),
-    in_pce_thread(emacs(Pos)).
+    location_url(Location, URL),        % File[:Line[:LinePos]]
+    run_pce_emacs(URL).
 do_edit_source(Location) :-             % External editor
     external_edit_command(Location, Command),
     print_message(informational, edit(waiting_for_editor)),
@@ -303,8 +326,7 @@ do_edit_source(Location) :-             % External editor
     ).
 
 external_edit_command(Location, Command) :-
-    memberchk(file(File), Location),
-    memberchk(line(Line), Location),
+    #{file:File, line:Line} :< Location,
     editor(Editor),
     file_base_name(Editor, EditorFile),
     file_name_extension(Base, _, EditorFile),
@@ -317,7 +339,7 @@ external_edit_command(Location, Command) :-
     !,
     atom_codes(Command, S).
 external_edit_command(Location, Command) :-
-    memberchk(file(File), Location),
+    #{file:File} :< Location,
     editor(Editor),
     file_base_name(Editor, EditorFile),
     file_name_extension(Base, _, EditorFile),
@@ -330,12 +352,19 @@ external_edit_command(Location, Command) :-
     !,
     atom_codes(Command, S).
 external_edit_command(Location, Command) :-
-    memberchk(file(File), Location),
+    #{file:File} :< Location,
     editor(Editor),
-    atomic_list_concat(['"', Editor, '" "', File, '"'], Command).
+    format(string(Command), '"~w" "~w"', [Editor, File]).
 
-pceemacs(pce_emacs).
-pceemacs(built_in).
+is_pceemacs(pce_emacs).
+is_pceemacs(built_in).
+
+%!  run_pce_emacs(+URL) is semidet.
+%
+%   Dynamically load and run emacs/1.
+
+run_pce_emacs(URL) :-
+    autoload_call(in_pce_thread(autoload_call(emacs(URL)))).
 
 %!  editor(-Editor)
 %
@@ -348,7 +377,7 @@ editor(Editor) :-                       % $EDITOR
         catch(getenv(Var, Editor), _, fail), !
     ;   Editor == default
     ->  catch(getenv('EDITOR', Editor), _, fail), !
-    ;   \+ pceemacs(Editor)
+    ;   \+ is_pceemacs(Editor)
     ->  !
     ).
 editor(Editor) :-                       % User defaults
@@ -407,39 +436,22 @@ substitute(_, _, Old, Old).
                  *            SELECT            *
                  *******************************/
 
-merge_locations(Pairs0, Pairs) :-
-    keysort(Pairs0, Pairs1),
-    merge_locations2(Pairs1, Pairs).
-
-merge_locations2([], []).
-merge_locations2([H0|T0], [H|T]) :-
-    remove_same_location(H0, H, T0, T1),
-    merge_locations2(T1, T).
-
-remove_same_location(Pair0, H, [Pair1|T0], L) :-
-    merge_locations(Pair0, Pair1, Pair2),
-    !,
-    remove_same_location(Pair2, H, T0, L).
-remove_same_location(H, H, L, L).
-
-merge_locations(Loc1-Spec1, Loc2-Spec2, Loc-Spec) :-
+merge_locations([L1|T1], Locations) :-
+    L1 = Loc1-Spec1,
+    select(L2, T1, T2),
+    L2 = Loc2-Spec2,
     same_location(Loc1, Loc2, Loc),
+    merge_specs(Spec1, Spec2, Spec),
     !,
-    (   merge_specs(Spec1, Spec2, Spec)
-    ;   merge_specs(Spec2, Spec1, Spec)
-    ;   Spec = Spec1
-    ),
-    !.
-merge_locations([file(X)]-_, Loc-Spec, Loc-Spec) :-
-    memberchk(file(X), Loc),
-    memberchk(line(_), Loc).
+    merge_locations([Loc-Spec|T2], Locations).
+merge_locations(Locations, Locations).
 
 same_location(L, L, L).
-same_location([file(F1)], [file(F2)], [file(F)]) :-
+same_location(#{file:F1}, #{file:F2}, #{file:F}) :-
     best_same_file(F1, F2, F).
-same_location([file(F1),line(L)], [file(F2)], [file(F),line(L)]) :-
+same_location(#{file:F1, line:Line}, #{file:F2}, #{file:F, line:Line}) :-
     best_same_file(F1, F2, F).
-same_location([file(F1)], [file(F2),line(L)], [file(F),line(L)]) :-
+same_location(#{file:F1}, #{file:F2, line:Line}, #{file:F, line:Line}) :-
     best_same_file(F1, F2, F).
 
 best_same_file(F1, F2, F) :-
@@ -452,9 +464,33 @@ best_same_file(F1, F2, F) :-
     ;   F = F2
     ).
 
-merge_specs(source_file(Path), _, source_file(Path)).
+merge_specs(Spec, Spec, Spec) :-
+    !.
+merge_specs(file(F1), file(F2), file(F)) :-
+    best_same_file(F1, F2, F),
+    !.
+merge_specs(Spec1, Spec2, Spec) :-
+    merge_specs_(Spec1, Spec2, Spec),
+    !.
+merge_specs(Spec1, Spec2, Spec) :-
+    merge_specs_(Spec2, Spec1, Spec),
+    !.
 
-%!  select_location(+Pairs, +UserSpec, -Location)
+merge_specs_(FileSpec, Spec, Spec) :-
+    is_filespec(FileSpec).
+
+is_filespec(source_file(_)) => true.
+is_filespec(Term),
+    compound(Term),
+    compound_name_arguments(Term, Alias, [_Arg]),
+    user:file_search_path(Alias, _) => true.
+is_filespec(_) =>
+    fail.
+
+%!  select_location(+Pairs, +UserSpec, -Location) is semidet.
+%
+%   @arg Pairs is a list of `Location-Spec` pairs
+%   @arg Location is a list of properties
 
 do_select_location(Pairs, Spec, Location) :-
     select_location(Pairs, Spec, Location),                % HOOK
@@ -464,45 +500,63 @@ do_select_location([], Spec, _) :-
     !,
     print_message(warning, edit(not_found(Spec))),
     fail.
-do_select_location([Location-_Spec], _, Location) :- !.
-do_select_location(Pairs, _, Location) :-
-    print_message(help, edit(select)),
-    list_pairs(Pairs, 0, N),
-    print_message(help, edit(prompt_select)),
-    read_number(N, I),
-    nth1(I, Pairs, Location-_Spec),
+do_select_location([#{file:File}-file(File)], _, Location) :-
+    !,
+    Location = #{file:File}.
+do_select_location([Location-_Spec], _, Location) :-
+    existing_location(Location),
     !.
+do_select_location(Pairs, _, Location) :-
+    foldl(number_location, Pairs, NPairs, 1, End),
+    print_message(help, edit(select(NPairs))),
+    (   End == 1
+    ->  fail
+    ;   Max is End - 1,
+        user_selection(Max, I),
+        memberchk(I-(Location-_Spec), NPairs)
+    ).
 
-list_pairs([], N, N).
-list_pairs([H|T], N0, N) :-
-    NN is N0 + 1,
-    list_pair(H, NN),
-    list_pairs(T, NN, N).
+%!  existing_location(+Location) is semidet.
+%
+%   True when Location can be edited.  By   default  that means that the
+%   file exists. This facility is hooked   to allow for alternative ways
+%   to reach the source, e.g., by lazily downloading it.
 
-list_pair(Pair, N) :-
-    print_message(help, edit(target(Pair, N))).
+existing_location(Location) :-
+    exists_location(Location),
+    !.
+existing_location(Location) :-
+    #{file:File} :< Location,
+    access_file(File, read).
 
+number_location(Pair, N-Pair, N, N1) :-
+    Pair = Location-_Spec,
+    existing_location(Location),
+    !,
+    N1 is N+1.
+number_location(Pair, 0-Pair, N, N).
+
+user_selection(Max, I) :-
+    user_select(Max, I),
+    !.
+user_selection(Max, I) :-
+    print_message(help, edit(choose(Max))),
+    read_number(Max, I).
+
+%!  read_number(+Max, -X) is semidet.
+%
+%   Read a number between 1 and Max. If Max < 10, use get_single_char/1.
 
 read_number(Max, X) :-
     Max < 10,
     !,
     get_single_char(C),
+    put_code(user_error, C),
     between(0'0, 0'9, C),
     X is C - 0'0.
 read_number(_, X) :-
-    read_line(Chars),
-    name(X, Chars),
-    integer(X).
-
-read_line(Chars) :-
-    get0(user_input, C0),
-    read_line(C0, Chars).
-
-read_line(10, []) :- !.
-read_line(-1, []) :- !.
-read_line(C, [C|T]) :-
-    get0(user_input, C1),
-    read_line(C1, T).
+    read_line_to_string(user_input, String),
+    number_string(X, String).
 
 
                  /*******************************
@@ -512,59 +566,93 @@ read_line(C, [C|T]) :-
 :- multifile
     prolog:message/3.
 
-prolog:message(edit(not_found(Spec))) -->
+prolog:message(edit(Msg)) -->
+    message(Msg).
+
+message(not_found(Spec)) -->
     [ 'Cannot find anything to edit from "~p"'-[Spec] ],
     (   { atom(Spec) }
     ->  [ nl, '    Use edit(file(~q)) to create a new file'-[Spec] ]
     ;   []
     ).
-prolog:message(edit(select)) -->
-    [ 'Please select item to edit:', nl, nl ].
-prolog:message(edit(prompt_select)) -->
+message(select(NPairs)) -->
+    { \+ (member(N-_, NPairs), N > 0) },
+    !,
+    [ 'Found the following locations:', nl ],
+    sequence(target, [nl], NPairs).
+message(select(NPairs)) -->
+    [ 'Please select item to edit:', nl ],
+    sequence(target, [nl], NPairs).
+message(choose(_Max)) -->
     [ nl, 'Your choice? ', flush ].
-prolog:message(edit(target(Location-Spec, N))) -->
-    [ '~t~d~3| '-[N]],
-    edit_specifier(Spec),
-    [ '~t~32|' ],
-    edit_location(Location).
-prolog:message(edit(waiting_for_editor)) -->
+message(waiting_for_editor) -->
     [ 'Waiting for editor ... ', flush ].
-prolog:message(edit(make)) -->
+message(make) -->
     [ 'Running make to reload modified files' ].
-prolog:message(edit(canceled)) -->
+message(canceled) -->
     [ 'Editor returned failure; skipped make/0 to reload files' ].
 
-edit_specifier(Module:Name/Arity) -->
-    !,
-    [ '~w:~w/~w'-[Module, Name, Arity] ].
-edit_specifier(file(_Path)) -->
-    !,
+target(0-(Location-Spec)) ==>
+    [ ansi(warning, '~t*~3| ', [])],
+    edit_specifier(Spec),
+    [ '~t~32|' ],
+    edit_location(Location, false),
+    [ ansi(warning, ' (no source available)', [])].
+target(N-(Location-Spec)) ==>
+    [ ansi(bold, '~t~d~3| ', [N])],
+    edit_specifier(Spec),
+    [ '~t~32|' ],
+    edit_location(Location, true).
+
+edit_specifier(Module:Name/Arity) ==>
+    [ '~w:'-[Module],
+      ansi(code, '~w/~w', [Name, Arity]) ].
+edit_specifier(file(_Path)) ==>
     [ '<file>' ].
-edit_specifier(source_file(_Path)) -->
-    !,
+edit_specifier(source_file(_Path)) ==>
     [ '<loaded file>' ].
-edit_specifier(include_file(_Path)) -->
-    !,
+edit_specifier(include_file(_Path)) ==>
     [ '<included file>' ].
-edit_specifier(Term) -->
+edit_specifier(Term) ==>
     [ '~p'-[Term] ].
 
-edit_location(Location) -->
-    { memberchk(file(File), Location),
-      memberchk(line(Line), Location),
-      short_filename(File, Spec)
+edit_location(Location, false) ==>
+    { location_label(Location, Label) },
+    [ ansi(warning, '~s', [Label]) ].
+edit_location(Location, true) ==>
+    { location_label(Location, Label),
+      location_url(Location, URL)
     },
+    [ url(URL, Label) ].
+
+location_label(Location, Label) :-
+    #{file:File, line:Line} :< Location,
     !,
-    [ '~q:~d'-[Spec, Line] ].
-edit_location(Location) -->
-    { memberchk(file(File), Location),
-      short_filename(File, Spec)
-    },
+    short_filename(File, ShortFile),
+    format(string(Label), '~w:~d', [ShortFile, Line]).
+location_label(Location, Label) :-
+    #{file:File} :< Location,
     !,
-    [ '~q'-[Spec] ].
+    short_filename(File, ShortFile),
+    format(string(Label), '~w', [ShortFile]).
+
+location_url(Location, File:Line:LinePos) :-
+    #{file:File, line:Line, linepos:LinePos} :< Location,
+    !.
+location_url(Location, File:Line) :-
+    #{file:File, line:Line} :< Location,
+    !.
+location_url(Location, File) :-
+    #{file:File} :< Location.
+
+%!  short_filename(+Path, -Spec) is det.
+%
+%   Spec is a way to refer to the file Path that is shorter. The path is
+%   shortened by either taking  it  relative   to  the  current  working
+%   directory or use one of the Prolog path aliases.
 
 short_filename(Path, Spec) :-
-    absolute_file_name('', Here),
+    working_directory(Here, Here),
     atom_concat(Here, Local0, Path),
     !,
     remove_leading_slash(Local0, Spec).
@@ -578,14 +666,16 @@ aliased_path(Path, Len-Spec) :-
     member(Alias, Aliases),
     Alias \== autoload,             % confusing and covered by something else
     Term =.. [Alias, '.'],
-    absolute_file_name(Term,
+    absolute_file_name(Term, Prefix,
                        [ file_type(directory),
                          file_errors(fail),
                          solutions(all)
-                       ], Prefix),
+                       ]),
     atom_concat(Prefix, Local0, Path),
-    remove_leading_slash(Local0, Local),
-    atom_length(Local, Len),
+    remove_leading_slash(Local0, Local1),
+    remove_extension(Local1, Local2),
+    unquote_segments(Local2, Local),
+    atom_length(Local2, Len),
     Spec =.. [Alias, Local].
 
 file_alias_path(Alias) :-
@@ -596,6 +686,32 @@ remove_leading_slash(Path, Local) :-
     !.
 remove_leading_slash(Path, Path).
 
+remove_extension(File0, File) :-
+    file_name_extension(File, Ext, File0),
+    user:prolog_file_type(Ext, source),
+    !.
+remove_extension(File, File).
+
+unquote_segments(File, Segments) :-
+    split_string(File, "/", "/", SegmentStrings),
+    maplist(atom_string, SegmentList, SegmentStrings),
+    maplist(no_quote_needed, SegmentList),
+    !,
+    segments(SegmentList, Segments).
+unquote_segments(File, File).
+
+
+no_quote_needed(A) :-
+    format(atom(Q), '~q', [A]),
+    Q == A.
+
+segments([Segment], Segment) :-
+    !.
+segments(List, A/Segment) :-
+    append(L1, [Segment], List),
+    !,
+    segments(L1, A).
+
 
                  /*******************************
                  *        LOAD EXTENSIONS       *
