@@ -995,8 +995,13 @@ class Prolog
     if ( args.length > 0 && typeof args[args.length-1] === "object" )
     { options = args.pop();
     }
+    const opts = {};
+    if ( options.engine )
+      opts.engine = options.engine;
     const module = options.module||"user";
-    return this.forEach("load_files(M:Files)", {M:module, Files:args});
+    return this.forEach("load_files(M:Files)",
+			{M:module, Files:args},
+			opts);
   }
 
   load_string(s, id)
@@ -1027,13 +1032,37 @@ class Prolog
     return name;
   }
 
+  /**
+   * Bind an event to a Prolog goal.
+   * @param {HTMLElement} elem Elem on which to listen for events
+   * @param {string} on Event type we listen for.  Passed to
+   * addEventListener().
+   * @param {string} goal Prolog goal to execute
+   * @param {object} [input] Optional input parameters for `goal`.
+   * @param {object} [options]
+   * @param {bool} [options.async] If `true`, run the handler
+   * asynchronously, i.e., using {@link Prolog#forEach}. Otherwise
+   * use {@link Prolog#query} to call the handler on the global
+   * Prolog engine.
+   */
 
-  bind(e, on, goal)
-  { const prolog = this;
+  bind(elem, on, goal, input, options) {
+    const prolog = this;
+    options = options||{};
 
-    e.addEventListener(on, (ev) =>
-    { prolog.query(goal, {Event__:ev}).once();
-    });
+    if ( options.async ) {
+      elem.addEventListener(on, async (ev) => {
+	prolog.forEach(goal,
+		       {...input, Event__:ev},
+		       {...options, engine:true});
+      });
+    } else {
+      elem.addEventListener(on, (ev) => {
+	prolog.query(goal,
+		     {...input, Event__:ev},
+		     options).once();
+      });
+    }
   }
 
   fetch(url, opts, type)
@@ -1131,6 +1160,8 @@ class Prolog
    * @param {Boolean} [options.engine] If `true`, run the goal in a
    * temporary engine.  Default is to use the current engine.  See
    * also Engine.query()
+   * @param {string} [options.string] Describes the target type for
+   * JavaScript strings.  One of `atom` or `string` (default)
    */
 
   query2(goal, input, options)
@@ -1143,7 +1174,10 @@ class Prolog
       input = input||{};
       options = options||{};
       prolog.put_chars(av+0, goal);
-      prolog.toProlog(input, av+1);
+      if ( options.string )
+	prolog.toProlog(input, av+1, {string:options.string});
+      else
+	prolog.toProlog(input, av+1);
 
       const q = new Query(prolog, 0, prolog.PL_Q_CATCH_EXCEPTION,
 			  "wasm_call_string/3", av,
@@ -1295,6 +1329,33 @@ class Prolog
     return new this.Promise(f);
   }
 
+  /**
+   * Create an abortable Promise that waits for an event on
+   * an HTMLElement.
+   *
+   * @param {HTMLElement} item The element on which the event is expected
+   * @param {string} eventType The event type as used by addEventListener()
+   * @return {Promise} A promise that is accepted if the event occurs
+   * and returns the event.  The promise can be aborted, in which case
+   * it is rejected.
+   */
+  promise_event(item, eventType) {
+    const f = function(resolve, reject) {
+      f.reject = reject;
+      const listener = (ev) => {
+	item.removeEventListener(eventType, listener);
+	resolve(ev);
+      }
+      item.addEventListener(eventType, listener);
+      f.abort = function() {
+	item.removeEventListener(eventType, listener);
+	f.reject("abort");
+      }
+    }
+
+    return new this.Promise(f);
+  }
+
 /**
  * @return {IOSTREAM*} as a number (pointer)
  */
@@ -1862,10 +1923,13 @@ class Prolog
     return rc;
   }
 
-  prompt_string(fd)
-  { const bytes = this.bindings.PL_prompt_string(fd||0)
-    if ( bytes )
-      return this.module.UTF8ToString(bytes);
+  prompt_string(fd) {
+    if ( fd == 0 ) {
+      iostream = this.stream("user_input");
+      const bytes = this.bindings.PL_prompt_string(iostream);
+      if ( bytes )
+	return this.module.UTF8ToString(bytes);
+    }
   }
 
 // If t is compound and index is between 1 and arity (inclusive),
@@ -2326,19 +2390,22 @@ function prolog_js_call(request, result)
 	}
       }
 
+      if ( obj == null || obj == undefined )
+	throw new TypeError(`${obj} has no attribute ${fname}`);
+
       const func = obj[fname];
       if ( typeof(func) === "function"  )
 	return func.apply(obj, args);
       else
-	console.log("ERROR: Function", fname, "is not defined on", obj);
+	throw new TypeError(`${obj}.${fname} is not a function`);
     }
 
-    for(let i=0; i<ar.length; i++)
-    { const next = ar[i];
+    for(let i=0; i<ar.length; i++) {
+      const next = ar[i];
 
-      if ( typeof(next) === "string" )
-      { if ( i == 0 )
-	{ switch(next)
+      if ( typeof(next) === "string" ) {
+	if ( i == 0 ) {
+	  switch(next)
 	  { case "prolog":
 	      obj = prolog;
 	      break;
@@ -2348,13 +2415,15 @@ function prolog_js_call(request, result)
 	    default:
 	      obj = eval(next);
 	  }
-	} else
-	{ obj = obj[next];
+	} else if ( obj == null || obj == undefined ) {
+	  throw new TypeError(`${obj} has no attribute ${next}`);
+	} else {
+	  obj = obj[next];
 	}
-      } else if ( next.v !== undefined )
-      { obj = next.v;
-      } else
-      { const args = next.args.map((v) => eval_chain(v));
+      } else if ( next.v !== undefined ) {
+	obj = next.v;
+      } else {
+	const args = next.args.map((v) => eval_chain(v));
 
 	obj = eval_one(obj, next.f, args);
       }
@@ -2363,28 +2432,31 @@ function prolog_js_call(request, result)
     return obj;
   }
 
-  try
-  { return prolog.with_frame(() =>
-    { const ar = prolog.toJSON(request, { string: "string" });
+  try {
+    return prolog.with_frame(() => {
+      const ar = prolog.toJSON(request, { string: "string" });
       let obj;
 
-      if ( ar.setter )
-      { const target = eval_chain(ar.target);
+      if ( ar.setter ) {
+	const target = eval_chain(ar.target);
 	const value  = eval_chain(ar.value);
 	target[ar.setter] = value;
 	obj = true;
-      } else
-      { obj = eval_chain(ar);
+      } else {
+	obj = eval_chain(ar);
       }
 
       return prolog.unify(result, prolog.toProlog(obj));
     }, false);
-  } catch (e)
-  { return prolog.bindings.PL_raise_exception(
-      prolog.toProlog(new prolog.Compound("error",
-					  [ new prolog.Compound("js_error", [e.toString()]),
-					    new prolog.Var()
-					  ])));
+  } catch (e) {
+    return prolog.bindings.PL_raise_exception(
+      prolog.toProlog(new prolog.Compound(
+	"error",
+	new prolog.Compound("js_eval_error",
+			    e.toString(),
+			    new prolog.Term(request)),
+	new prolog.Var()
+	)));
   }
 }
 
